unit UFormNew;

{$mode delphi}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ComCtrls, Spin, ExtCtrls, ActnList, ExtDlgs, Menus, UGame, UGeometry,
  UGameServer, UServerList;

type

  { TFormNew }

  TFormNew = class(TForm)
    AServerModify: TAction;
    AServerAdd: TAction;
    AServerRemove: TAction;
    APlayerAdd: TAction;
    APlayerRemove: TAction;
    APlayerModify: TAction;
    ActionList1: TActionList;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    ButtonCancel: TButton;
    ButtonImageBrowse: TButton;
    ButtonOk: TButton;
    ButtonPlayerAdd: TButton;
    ButtonPlayerModify: TButton;
    ButtonPlayerRemove: TButton;
    CheckBoxBridges: TCheckBox;
    CheckBoxFogOfWar: TCheckBox;
    CheckBoxCity: TCheckBox;
    CheckBoxSymetricMap: TCheckBox;
    CheckBoxVoid: TCheckBox;
    ComboBoxGridType: TComboBox;
    ComboBoxMapShape: TComboBox;
    ComboBoxWinObjective: TComboBox;
    EditServerAddress: TEdit;
    EditImageFile: TEdit;
    Label1: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    ListViewPlayers: TListView;
    ListViewServers: TListView;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItem5: TMenuItem;
    MenuItem6: TMenuItem;
    OpenPictureDialog1: TOpenPictureDialog;
    PageControl1: TPageControl;
    PageControl2: TPageControl;
    PanelButtons: TPanel;
    PanelChat: TPanel;
    PopupMenuPlayers: TPopupMenu;
    PopupMenuServers: TPopupMenu;
    RadioButtonModeNetworkServer: TRadioButton;
    RadioButtonModeNetworkClient: TRadioButton;
    RadioButtonModeLocal: TRadioButton;
    RadioGroupGrowAmount: TRadioGroup;
    RadioGroupGrowCells: TRadioGroup;
    SpinEditMaxPower: TSpinEdit;
    SpinEditServerPort: TSpinEdit;
    SpinEditCityPercent: TSpinEdit;
    SpinEditMapSizeX: TSpinEdit;
    SpinEditMapSizeY: TSpinEdit;
    SpinEditNeutralUnits: TSpinEdit;
    SpinEditSpecialCells: TSpinEdit;
    SpinEditTurns: TSpinEdit;
    SpinEditVoidPercent: TSpinEdit;
    Splitter1: TSplitter;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    TabSheet3: TTabSheet;
    TabSheetStayAliveTurns: TTabSheet;
    TabSheetCaptureCells: TTabSheet;
    TabSheetDefeatEnemies: TTabSheet;
    TabSheetCaptureCities: TTabSheet;
    TabSheetMode: TTabSheet;
    TrackBarSizeX: TTrackBar;
    TrackBarSizeY: TTrackBar;
    procedure APlayerAddExecute(Sender: TObject);
    procedure APlayerModifyExecute(Sender: TObject);
    procedure APlayerRemoveExecute(Sender: TObject);
    procedure AServerAddExecute(Sender: TObject);
    procedure AServerModifyExecute(Sender: TObject);
    procedure AServerRemoveExecute(Sender: TObject);
    procedure ButtonImageBrowseClick(Sender: TObject);
    procedure CheckBoxCityChange(Sender: TObject);
    procedure CheckBoxVoidChange(Sender: TObject);
    procedure ComboBoxMapShapeChange(Sender: TObject);
    procedure ComboBoxWinObjectiveChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ListViewPlayersCustomDrawSubItem(Sender: TCustomListView;
      Item: TListItem; SubItem: Integer; State: TCustomDrawState;
      var DefaultDraw: Boolean);
    procedure ListViewPlayersData(Sender: TObject; Item: TListItem);
    procedure ListViewPlayersDblClick(Sender: TObject);
    procedure ListViewPlayersKeyPress(Sender: TObject; var Key: char);
    procedure ListViewPlayersKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState
      );
    procedure ListViewPlayersSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure ListViewServersData(Sender: TObject; Item: TListItem);
    procedure ListViewServersKeyPress(Sender: TObject; var Key: char);
    procedure ListViewServersSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure RadioButtonModeLocalChange(Sender: TObject);
    procedure SpinEditMapSizeXChange(Sender: TObject);
    procedure SpinEditMapSizeYChange(Sender: TObject);
    procedure SpinEditMaxPowerChange(Sender: TObject);
    procedure TrackBarSizeYChange(Sender: TObject);
    procedure TrackBarSizeXChange(Sender: TObject);
  private
    Players: TPlayers;
    ServerList: TServerList;
  public
    procedure Translate;
    procedure ReloadView;
    procedure UpdateInterface;
    procedure Load(Server: TServer);
    procedure Save(Server: TServer);
  end;

var
  FormNew: TFormNew;

implementation

{$R *.lfm}

uses
  UFormPlayer, UFormChat, UCore, UFormServer;

resourcestring
  SGridTypeHexagon = 'Hexagonal';
  SGridTypeSquare = 'Square';
  SGridTypeTriangle = 'Triangonal';
  SGridTypeVoronoi = 'Voronoi random';
  SGridTypeIsometric = 'Isometric';
  SWinObjectiveDefeatAllOponents = 'Defeat all oponents';
  SWinObjectiveDefeatAllCities = 'Defeat all oponents cities';
  SWinObjectiveCapturePosition = 'Capture all special cells';
  SWinObjectiveStayAliveForTurns = 'Stay alive for number of turns';
  SGrowNone = 'None';
  SGrowPlayerCities = 'Player cities';
  SGrowPlayerAllCells = 'Player all cells';
  SGrowAmountByOne = 'By one';
  SGrowAmountBySquareRoot = 'By square root';
  SRectangular = 'Rectangular';
  SRounded = 'Rounded';
  SFromImageFile = 'From image file';
  SRemoveServer = 'Remove server';
  SRemoveServerQuery = 'Do you want to remove server?';

{ TFormNew }

procedure TFormNew.ListViewPlayersData(Sender: TObject; Item: TListItem);
begin
  if (Item.Index >= 0) and (Item.Index < Players.Count) then
  with Players[Item.Index] do begin
    Item.Caption := Name;
    Item.Data := Players[Item.Index];
    Item.SubItems.Add(PlayerModeText[Mode]);
    Item.SubItems.Add(''); // Do not show color text
    Item.SubItems.Add(IntToStr(StartUnits));
  end;
end;

procedure TFormNew.ListViewPlayersDblClick(Sender: TObject);
begin
  APlayerModify.Execute;
end;

procedure TFormNew.ListViewPlayersKeyPress(Sender: TObject; var Key: char);
begin
  if (Key = #13) then APlayerModify.Execute;
end;

procedure TFormNew.ListViewPlayersKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = 45 then APlayerAdd.Execute;
  if Key = 46 then APlayerRemove.Execute;
end;

procedure TFormNew.ListViewPlayersSelectItem(Sender: TObject; Item: TListItem;
  Selected: Boolean);
begin
  ReloadView;
end;

procedure TFormNew.ListViewServersData(Sender: TObject; Item: TListItem);
begin
  if Item.Index < ServerList.Items.Count then
  with TServerInfo(ServerList.Items[Item.Index]) do begin
    Item.Caption := Name;
    Item.Data := ServerList.Items[Item.Index];
    Item.SubItems.Add(Address + ':' + IntToStr(Port));
  end;
end;

procedure TFormNew.ListViewServersKeyPress(Sender: TObject; var Key: char);
begin
  if Key = #13 then AServerModify.Execute;
end;

procedure TFormNew.ListViewServersSelectItem(Sender: TObject; Item: TListItem;
  Selected: Boolean);
begin
  UpdateInterface;
end;

procedure TFormNew.RadioButtonModeLocalChange(Sender: TObject);
begin
  UpdateInterface;
end;

procedure TFormNew.SpinEditMapSizeXChange(Sender: TObject);
begin
  TrackBarSizeX.Position := SpinEditMapSizeX.Value;
end;

procedure TFormNew.SpinEditMapSizeYChange(Sender: TObject);
begin
  TrackBarSizeY.Position := SpinEditMapSizeY.Value;
end;

procedure TFormNew.SpinEditMaxPowerChange(Sender: TObject);
begin
  UpdateInterface;
end;

procedure TFormNew.TrackBarSizeYChange(Sender: TObject);
begin
  SpinEditMapSizeY.Value := TrackBarSizeY.Position;
end;

procedure TFormNew.TrackBarSizeXChange(Sender: TObject);
begin
  SpinEditMapSizeX.Value := TrackBarSizeX.Position;
end;

procedure TFormNew.Translate;
var
  LastIndex: Integer;
begin
  with ComboBoxGridType do begin
    LastIndex := ItemIndex;
    Clear;
    Items.Add(SGridTypeHexagon);
    Items.Add(SGridTypeSquare);
    Items.Add(SGridTypeTriangle);
    Items.Add(SGridTypeVoronoi);
    Items.Add(SGridTypeIsometric);
    ItemIndex := LastIndex;
  end;
  with ComboBoxWinObjective do begin
    LastIndex := ItemIndex;
    Clear;
    Items.Add(SWinObjectiveDefeatAllOponents);
    Items.Add(SWinObjectiveDefeatAllCities);
    Items.Add(SWinObjectiveCapturePosition);
    Items.Add(SWinObjectiveStayAliveForTurns);
    ItemIndex := LastIndex;
  end;
  with ComboBoxMapShape do begin
    LastIndex := ItemIndex;
    Clear;
    Items.Add(SRectangular);
    Items.Add(SFromImageFile);
    Items.Add(SRounded);
    ItemIndex := LastIndex;
  end;
  with RadioGroupGrowCells do begin
    LastIndex := ItemIndex;
    Items.Clear;
    Items.Add(SGrowNone);
    Items.Add(SGrowPlayerCities);
    Items.Add(SGrowPlayerAllCells);
    ItemIndex := LastIndex;
  end;
  with RadioGroupGrowAmount do begin
    LastIndex := ItemIndex;
    Items.Clear;
    Items.Add(SGrowAmountByOne);
    Items.Add(SGrowAmountBySquareRoot);
    ItemIndex := LastIndex;
  end;
  Core.CoolTranslator1.TranslateComponentRecursive(Self);
end;

procedure TFormNew.ReloadView;
begin
  ListViewPlayers.Items.Count := Players.Count;
  ListViewPlayers.Refresh;
  ListViewServers.Items.Count := ServerList.Items.Count;
  ListViewServers.Refresh;
  Translate;
  UpdateInterface;
end;

procedure TFormNew.UpdateInterface;
begin
  APlayerRemove.Enabled := Assigned(ListViewPlayers.Selected) and (Players.Count > 2);
  APlayerAdd.Enabled := Players.Count < MaxPlayerCount;
  APlayerModify.Enabled := Assigned(ListViewPlayers.Selected);
  EditImageFile.Enabled := ComboBoxMapShape.ItemIndex = Integer(msImage);
  ButtonImageBrowse.Enabled := ComboBoxMapShape.ItemIndex = Integer(msImage);
  EditServerAddress.Enabled := RadioButtonModeNetworkServer.Checked;
  SpinEditServerPort.Enabled := RadioButtonModeNetworkServer.Checked;
  ListViewServers.Enabled := RadioButtonModeNetworkClient.Checked;
  PanelChat.Visible := RadioButtonModeNetworkClient.Checked or RadioButtonModeNetworkServer.Checked;
  SpinEditNeutralUnits.MaxValue := SpinEditMaxPower.Value;
  AServerRemove.Enabled := RadioButtonModeNetworkClient.Checked and Assigned(ListViewServers.Selected);
  AServerAdd.Enabled := RadioButtonModeNetworkClient.Checked;
  AServerModify.Enabled := RadioButtonModeNetworkClient.Checked;
  PageControl2.TabIndex := ComboBoxWinObjective.ItemIndex;
end;

procedure TFormNew.FormShow(Sender: TObject);
begin
  Core.PersistentForm.Load(Self);
  ReloadView;
  //Height := Trunc(1.5 * Height);
  //PageControl1.TabIndex := 0;
end;

procedure TFormNew.ListViewPlayersCustomDrawSubItem(Sender: TCustomListView;
  Item: TListItem; SubItem: Integer; State: TCustomDrawState;
  var DefaultDraw: Boolean);
begin
  if SubItem = 2 then
  with ListViewPlayers.Canvas do begin
    Brush.Color := TPlayer(Item.Data).Color;
    Brush.Style := bsSolid;
    FillRect(Item.DisplayRectSubItem(2, drBounds));
  end;
end;

procedure TFormNew.FormCreate(Sender: TObject);
begin
  Players := TPlayers.Create;
  ServerList := TServerList.Create;
  FormChat := TFormChat.Create(nil);
  FormChat.ManualDock(PanelChat, nil, alClient);
  FormChat.Align := alClient;
  FormChat.Show;
  Translate;
end;

procedure TFormNew.CheckBoxCityChange(Sender: TObject);
begin
  SpinEditCityPercent.Enabled := CheckBoxCity.Checked;
end;

procedure TFormNew.CheckBoxVoidChange(Sender: TObject);
begin
  SpinEditVoidPercent.Enabled := CheckBoxVoid.Checked;
end;

procedure TFormNew.ComboBoxMapShapeChange(Sender: TObject);
begin
  ReloadView;
end;

procedure TFormNew.ComboBoxWinObjectiveChange(Sender: TObject);
begin
  UpdateInterface;
end;

procedure TFormNew.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  Core.PersistentForm.Save(Self);
end;

procedure TFormNew.APlayerModifyExecute(Sender: TObject);
begin
  FormPlayer := TFormPlayer.Create(nil);
  try
    if Assigned(ListViewPlayers.Selected) then begin
      FormPlayer.Load(TPlayer(ListViewPlayers.Selected.Data));
       if FormPlayer.ShowModal = mrOk then begin
         FormPlayer.Save(TPlayer(ListViewPlayers.Selected.Data));
         ReloadView;
      end;
    end;
  finally
    FormPlayer.Free;
  end;
end;

procedure TFormNew.APlayerRemoveExecute(Sender: TObject);
begin
  if Assigned(ListViewPlayers.Selected) then begin
    Players.Remove(ListViewPlayers.Selected.Data);
    ReloadView;
  end;
end;

procedure TFormNew.AServerAddExecute(Sender: TObject);
var
  NewServer: TServerInfo;
begin
  FormServer := TFormServer.Create(nil);
  NewServer := TServerInfo.Create;
  FormServer.Load(NewServer);
  if FormServer.ShowModal = mrOK then begin
    FormServer.Save(NewServer);
    ServerList.Items.Add(NewServer);
    ReloadView;
  end else NewServer.Free;
  FormServer.Free;
end;

procedure TFormNew.AServerModifyExecute(Sender: TObject);
var
  Server: TServerInfo;
begin
  FormServer := TFormServer.Create(nil);
  Server := TServerInfo(ListViewServers.Selected.Data);
  FormServer.Load(Server);
  if FormServer.ShowModal = mrOK then begin
    FormServer.Save(Server);
    ReloadView;
  end;
  FormServer.Free;
end;

procedure TFormNew.AServerRemoveExecute(Sender: TObject);
var
  I: Integer;
begin
  if MessageDlg(SRemoveServer, SRemoveServerQuery,
  TMsgDlgType.mtConfirmation, [mbCancel, mbOk], 0) = mrOk then begin
    for I := ListViewServers.Items.Count - 1 downto 0 do
    if ListViewServers.Items[I].Selected then
      ServerList.Items.Delete(I);
    ReloadView;
  end;
end;

procedure TFormNew.ButtonImageBrowseClick(Sender: TObject);
begin
  OpenPictureDialog1.FileName := EditImageFile.Text;
  if OpenPictureDialog1.Execute then begin
    EditImageFile.Text := OpenPictureDialog1.FileName;
  end;
end;

procedure TFormNew.APlayerAddExecute(Sender: TObject);
var
  NewPlayer: TPlayer;
begin
  if Players.Count <= (MaxPlayerCount - 1) then begin
    NewPlayer := TPlayer.Create;
    NewPlayer.Id := Players.GetNewPlayerId;
    NewPlayer.Name := SPlayer + ' ' + IntToStr(NewPlayer.Id);
    NewPlayer.Color := PlayerColors[Players.Count];
    NewPlayer.Mode := pmComputer;
    NewPlayer.Agressivity := caMedium;
    Players.Add(NewPlayer);
    ReloadView;
  end;
end;

procedure TFormNew.FormDestroy(Sender: TObject);
begin
  FreeAndNil(ServerList);
  FreeAndNil(Players);
  FreeAndNil(FormChat);
end;

procedure TFormNew.Load(Server: TServer);
begin
  with Server do begin
    RadioButtonModeLocal.Checked := Mode = smLocal;
    RadioButtonModeNetworkClient.Checked := Mode = smNetworkClient;
    RadioButtonModeNetworkServer.Checked := Mode = smNetworkServer;
    EditServerAddress.Text := LocalNetworkAddress;
    SpinEditServerPort.Value := LocalNetworkPort;
  end;
  ServerList.Assign(Core.ServerList);
  Players.Assign(Core.GameSettings.Players);
  with Core.GameSettings do begin
    CheckBoxSymetricMap.Checked := SymetricMap;
    CheckBoxVoid.Checked := VoidEnabled;
    SpinEditVoidPercent.Value := VoidPercentage;
    SpinEditVoidPercent.Enabled := VoidEnabled;
    CheckBoxCity.Checked := CityEnabled;
    SpinEditCityPercent.Value := CityPercentage;
    SpinEditMapSizeX.Value := Map.Size.X;
    SpinEditMapSizeY.Value := Map.Size.Y;
    TrackBarSizeX.Position := Map.Size.X;
    TrackBarSizeY.Position := Map.Size.Y;
    RadioGroupGrowAmount.ItemIndex := Integer(GrowAmount);
    RadioGroupGrowCells.ItemIndex := Integer(GrowCells);
    ComboBoxGridType.ItemIndex := Integer(MapType) - 1;
    ComboBoxWinObjective.ItemIndex := Integer(WinObjective);
    SpinEditNeutralUnits.Value := MaxNeutralUnits;
    ComboBoxMapShape.ItemIndex := Integer(Map.Shape);
    EditImageFile.Text := MapImageFileName;
    CheckBoxFogOfWar.Checked := FogOfWar;
    CheckBoxBridges.Checked := BridgeEnabled;
    SpinEditMaxPower.Value := MaxPower;
    SpinEditTurns.Value := StayAliveForDefinedTurns;
    SpinEditSpecialCells.Value := SpecialCaptureCellCount;
  end;
end;

procedure TFormNew.Save(Server: TServer);
begin
  with Server do begin
    if RadioButtonModeLocal.Checked then Mode := smLocal
    else if RadioButtonModeNetworkClient.Checked then Mode := smNetworkClient
    else if RadioButtonModeNetworkServer.Checked then Mode := smNetworkServer;
    LocalNetworkAddress := EditServerAddress.Text;
    LocalNetworkPort := SpinEditServerPort.Value;
    if Assigned(ListViewServers.Selected) then begin
      RemoteNetworkAddress := TServerInfo(ListViewServers.Selected.Data).Address;
      RemoteNetworkPort := TServerInfo(ListViewServers.Selected.Data).Port;
    end else begin
      RemoteNetworkAddress := '';
      RemoteNetworkPort := 0;
    end;
  end;
  Core.GameSettings.Players.Assign(Players);
  Core.ServerList.Assign(ServerList);
  with Core.GameSettings do begin
    SymetricMap := CheckBoxSymetricMap.Checked;
    VoidEnabled := CheckBoxVoid.Checked;
    VoidPercentage := SpinEditVoidPercent.Value;
    CityEnabled := CheckBoxCity.Checked;
    CityPercentage := SpinEditCityPercent.Value;
    Map.Size := TPoint.Create(SpinEditMapSizeX.Value, SpinEditMapSizeY.Value);
    GrowAmount := TGrowAmount(RadioGroupGrowAmount.ItemIndex);
    GrowCells := TGrowCells(RadioGroupGrowCells.ItemIndex);
    MapType := TMapType(ComboBoxGridType.ItemIndex + 1);
    WinObjective := TWinObjective(ComboBoxWinObjective.ItemIndex);
    MaxNeutralUnits := SpinEditNeutralUnits.Value;
    Map.Shape := TMapShape(ComboBoxMapShape.ItemIndex);
    MapImageFileName := EditImageFile.Text;
    FogOfWar := CheckBoxFogOfWar.Checked;
    BridgeEnabled := CheckBoxBridges.Checked;
    MaxPower := SpinEditMaxPower.Value;
    StayAliveForDefinedTurns := SpinEditTurns.Value;
    SpecialCaptureCellCount := SpinEditSpecialCells.Value;
  end;
end;

end.

